@@ -1,8 +1,9 @@
-package Pcore::PDF v0.3.7;
+package Pcore::PDF v0.4.0;
 
 use Pcore -dist, -class, -const, -result;
 use Config;
 use Pcore::Util::Data qw[to_json from_json];
+use Pcore::Util::Scalar qw[is_plain_scalarref];
 
 const our $PAGE_SIZE => {
     A0        => '841 x 1189 mm',
@@ -94,8 +95,55 @@ has _queue => ( is => 'ro', isa => ArrayRef, default => sub { [] }, init_arg =>
 #     'job-resource-count' => 1,
 # };
 
+# TODO MSWIN support
+sub update_all ( $self, $cb = undef ) {
+    die qq[MSWIN is not supported] if $MSWIN;
+
+    my $blocking_cv = defined wantarray ? AE::cv : undef;
+
+    my $princexml_ver = $ENV->dist('Pcore-PDF')->cfg->{princexml_ver};
+
+    my $url = "https://www.princexml.com/download/prince-$princexml_ver" . ( $MSWIN ? '-win64.zip' : '-linux-generic-x86_64.tar.gz' );
+
+    P->http->get(
+        $url,
+        buf_size    => 1,
+        on_progress => 1,
+        sub ($res) {
+            my $success;
+
+            if ($res) {
+                $success = eval {
+                    if ($MSWIN) {
+
+                        # TODO
+                    }
+                    else {
+                        P->file->untar( $res->body->path, $ENV->share->get_storage( 'bin/', 'Pcore-PDF' ) . "princexml-linux-generic-x64-$princexml_ver/", strip_component => 1 );
+                    }
+
+                    1;
+                };
+            }
+
+            $cb->($success) if $cb;
+
+            $blocking_cv->($success) if $blocking_cv;
+
+            return;
+        }
+    );
+
+    return $blocking_cv ? $blocking_cv->recv : ();
+}
+
+# TODO MSWIN support
 sub _build_prince ($self) {
-    my $path = '/bin/princexml' . ( $MSWIN ? '-win' : '-linux' ) . ( $Config{archname} =~ /x64|x86_64/sm ? '-x64' : q[] ) . "-v@{[$ENV->dist('Pcore-PDF')->cfg->{princexml_ver}]}/" . ( $MSWIN ? 'bin/prince.exe' : 'bin/prince' );
+    my $princexml_ver = $ENV->dist('Pcore-PDF')->cfg->{princexml_ver};
+
+    my $path = '/bin/princexml-linux-generic' . ( $Config{archname} =~ /x64|x86_64/sm ? '-x64' : q[] ) . "-$princexml_ver/lib/prince/bin/prince";
+
+    say $path;
 
     return $ENV->share->get($path);
 }
@@ -263,7 +311,7 @@ sub _run_task ( $self, $proc, $task ) {
 
     my ( $job, $resources );
 
-    if ( ref $task->[0] ) {
+    if ( is_plain_scalarref $task->[0] ) {
         $job->{input}->{src} = 'job-resource:0';
 
         push $resources->@*, $task->[0];
@@ -373,10 +421,12 @@ sub _run_task ( $self, $proc, $task ) {
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
+## |    3 | 100                  | ValuesAndExpressions::ProhibitInterpolationOfLiterals - Useless interpolation of literal string                |
+## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
 ## |    2 |                      | Documentation::RequirePodLinksIncludeText                                                                      |
-## |      | 384                  | * Link L<Pcore::Util::Result> on line 441 does not specify text                                                |
-## |      | 384                  | * Link L<Pcore::Util::Result> on line 451 does not specify text                                                |
-## |      | 384                  | * Link L<Pcore> on line 449 does not specify text                                                              |
+## |      | 434                  | * Link L<Pcore::Util::Result> on line 491 does not specify text                                                |
+## |      | 434                  | * Link L<Pcore::Util::Result> on line 501 does not specify text                                                |
+## |      | 434                  | * Link L<Pcore> on line 499 does not specify text                                                              |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
